import React, { useState, ChangeEvent } from "react";
import { useDispatch, useSelector } from "react-redux";
import { closeJob, createJob, updateJob } from "../../common/actions/common";
import { getUserId } from "../../common/lib/util";
import { JobType } from "../../common/types";
import CustomModal from "./Modal";

const initialJob: JobType = {
  title: "",
  description: "",
  primarySkills: [],
  secondarySkills: [],
  salary: {
    min: 0,
    max: 0,
    currency: "LKR",
  },
  organization: "",
};

type OwnProps = {
  mode: "create" | "update";
  defaultJob?: JobType;
};

const CreateUpdateJob = ({
  mode = "create",
  defaultJob = initialJob,
}: OwnProps) => {
  const dispatch = useDispatch();
  const userId = useSelector(getUserId);
  const [isLoading, setIsLoading] = useState<boolean>(false);
  const [isOpen, setIsOpen] = useState<boolean>(false);
  const [job, setJob] = useState<JobType>(defaultJob);
  const [primarySkill, setPrimarySkill] = useState<string>("");
  const [secondarySkill, setSecondarySkill] = useState<string>("");

  const toggleModal = () => setIsOpen(!isOpen);

  const onChangeSkills = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.name === "primary") {
      setPrimarySkill(e.target.value);
    } else {
      setSecondarySkill(e.target.value);
    }
  };

  const onChangeJob = (
    e: ChangeEvent<HTMLInputElement | HTMLTextAreaElement>
  ) => {
    setJob({ ...job, [e.target.name]: e.target.value });
  };

  const onChangeSalary = (e: ChangeEvent<HTMLInputElement>) => {
    setJob({
      ...job,
      salary: { ...job.salary, [e.target.name]: e.target.value },
    });
  };

  const onAddSkill = (type: "primary" | "secondary") => {
    if (type === "primary") {
      if (primarySkill.length > 0) {
        setJob({
          ...job,
          primarySkills: [...job.primarySkills, primarySkill],
        });
        setPrimarySkill("");
      }
    } else {
      if (secondarySkill.length > 0) {
        setJob({
          ...job,
          secondarySkills: [...job.secondarySkills, secondarySkill],
        });
        setSecondarySkill("");
      }
    }
  };

  const removeSkill = (type: string, index: number) => {
    let key: "primarySkills" | "secondarySkills" = "secondarySkills";
    if (type === "primary") {
      key = "primarySkills";
    }
    const skills = job[key].filter((_, i) => i !== index);
    setJob({ ...job, [key]: [...skills] });
  };

  const renderPrimarySkills = (skill: string, index: number) => {
    const onClick = () => removeSkill("primary", index);
    return (
      <span className="skill" key={index}>
        {skill}{" "}
        <span className="span-btn" onClick={onClick}>
          X
        </span>
      </span>
    );
  };

  const renderSecondarykills = (skill: string, index: number) => {
    const onClick = () => removeSkill("secondary", index);
    return (
      <span className="skill secondary" key={index}>
        {skill}{" "}
        <span className="span-btn" onClick={onClick}>
          X
        </span>
      </span>
    );
  };

  const onSubmit = () => {
    const {
      title,
      description,
      salary: { min, max },
      primarySkills,
    } = job;
    if (
      title.length > 0 &&
      description.length > 0 &&
      primarySkills.length > 0 &&
      min > 0 &&
      max > 0
    ) {
      const payload: JobType = {
        ...job,
        salary: {
          currency: job.salary.currency,
          max: Number(max),
          min: Number(min),
        },
        organization: userId,
      };
      setIsLoading(true);
      if (mode === "create") {
        dispatch(createJob(payload, callback));
      } else {
        dispatch(updateJob(payload, callback));
      }
    }
  };

  const callback = () => {
    setIsLoading(false);
    toggleModal();
  };

  const onCloseJob = () => {
    dispatch(closeJob(defaultJob?._id || ""));
  };

  const btnText = mode === "create" ? "Create job" : "Update job";

  return (
    <div className="mt-3">
      <button
        className="btn btn-primary"
        style={{ width: "100%" }}
        onClick={toggleModal}
      >
        {btnText}
      </button>
      <CustomModal
        isOpen={isOpen}
        onRequestClose={toggleModal}
        size="lg"
        shouldCloseOnOverlayClick={false}
      >
        <h4>{mode === "create" ? " Create new job" : "Edit job"}</h4>
        <div className="mb-3">
          <label className="form-label">Job title</label>
          <input
            type="text"
            className="form-control"
            name="title"
            onChange={onChangeJob}
            value={job.title}
          />
        </div>
        <div className="mb-3">
          <label className="form-label">Job description</label>
          <textarea
            className="form-control"
            rows={5}
            name="description"
            onChange={onChangeJob}
            value={job.description}
          />
        </div>
        <div className="mb-2">
          <label className="form-label">Primary skills</label>
          <div className="input-group mb-3">
            <input
              type="text"
              className="form-control"
              placeholder=""
              name="primary"
              onChange={onChangeSkills}
              value={primarySkill}
            />
            <button
              className="btn btn-secondary"
              type="button"
              id="button-addon1"
              onClick={() => onAddSkill("primary")}
            >
              Add
            </button>
          </div>
        </div>
        <div className="skills">
          {job.primarySkills.map(renderPrimarySkills)}
        </div>
        <div className="mb-2">
          <label className="form-label">Secondary skills (optional)</label>
          <div className="input-group mb-3">
            <input
              type="text"
              className="form-control"
              placeholder=""
              name="secondary"
              onChange={onChangeSkills}
              value={secondarySkill}
            />
            <button
              className="btn btn-secondary"
              type="button"
              id="button-addon1"
              onClick={() => onAddSkill("secondary")}
            >
              Add
            </button>
          </div>
        </div>
        <div className="skills">
          {job.secondarySkills.map(renderSecondarykills)}{" "}
        </div>
        <div className="mb-3">
          <label className="form-label">Salary Range</label>
          <div className="input-group mb-3">
            <input
              type="number"
              className="form-control"
              placeholder="min"
              name="min"
              onChange={onChangeSalary}
              value={job.salary.min}
            />
            <span className="input-group-text">-</span>
            <input
              type="number"
              className="form-control"
              placeholder="max"
              name="max"
              onChange={onChangeSalary}
              value={job.salary.max}
            />
            <span className="input-group-text">-</span>
            <input
              type="text"
              className="form-control"
              placeholder="currency"
              name="currency"
              onChange={onChangeSalary}
              value={job.salary.currency}
            />
          </div>
        </div>
        <div className="d-md-block">
          {mode !== "create" && (
            <button
              className="btn btn-secondary m-2"
              onClick={onCloseJob}
              disabled={isLoading}
            >
              Close Job
            </button>
          )}
          <div style={{ float: "right" }}>
            <button
              className="btn btn-primary m-2"
              onClick={onSubmit}
              disabled={isLoading}
            >
              {btnText}
            </button>
            <button className="btn btn-danger" onClick={toggleModal}>
              Cancel
            </button>
          </div>
        </div>
      </CustomModal>
    </div>
  );
};

export default CreateUpdateJob;
